package dev.ninjdai.werewolf.uhc.roles;

import dev.ninjdai.werewolf.uhc.UHCPlayer;
import dev.ninjdai.werewolf.uhc.effects.Effect;
import dev.ninjdai.werewolf.uhc.events.TimedPlayerDeath;
import lombok.Getter;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.Style;
import net.kyori.adventure.text.format.TextColor;
import net.kyori.adventure.text.format.TextDecoration;
import net.minestom.server.item.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.*;

public class Role {
    public Role(String id, Component name, List<ItemStack> roleItems, List<Effect> roleEffects, Set<RoleProperties> roleProperties, boolean isDuo, int roleMaxCount, Component description) {
        this.id = id;
        this.name = name;
        this.roleItems = roleItems;
        this.roleEffects = roleEffects;
        this.roleProperties = roleProperties;
        this.isDuo = isDuo;
        this.roleMaxCount = roleMaxCount;
        this.description = description;

        roleMap.put(this.id, this);
    }

    Role(@NotNull Builder builder) {
        this.id = builder.id;
        this.name = builder.name;
        this.roleItems = builder.roleItems;
        this.roleEffects = builder.roleEffects;
        this.roleProperties = builder.roleProperties;
        this.isDuo = builder.isDuo;
        this.roleMaxCount = builder.roleMaxCount;
        this.description = builder.description;

        roleMap.put(this.id, this);
    }

    @Getter private String id;
    @Getter private Component name;
    @Getter private List<ItemStack> roleItems;
    @Getter private List<Effect> roleEffects;
    @Getter private Set<RoleProperties> roleProperties;
    @Getter private boolean isDuo;
    @Getter private int roleMaxCount;

    @Getter private Component description;

    public void onPlayerDeath(@NotNull UHCPlayer player, TimedPlayerDeath timedPlayerDeath) {
        player.sendMessage(Component.text(
                "You died, but don't leave too soon ! You may be resurrected in the next 15 seconds.\nYou can still watch the game after dying definitely.",
                Style.style(TextColor.color(184,14,14), TextDecoration.ITALIC)
        ));
    }

    // ID: Role
    public static final Map<String, Role> roleMap = new HashMap<>();

    public static class Builder {
        private String id = "DUMMY";
        private Component name = Component.text("Dummy");
        private List<ItemStack> roleItems = new ArrayList<>();
        private List<Effect> roleEffects = new ArrayList<>();
        private Set<RoleProperties> roleProperties = new HashSet<>();
        private boolean isDuo = false;
        private int roleMaxCount = 1;
        private Component description = Component.text("");

        public Builder setId(String id) {
            this.id = id;
            return this;
        }

        public Builder setName(Component name) {
            this.name = name;
            return this;
        }

        public Builder setDescription(Component description) {
            this.description = description;
            return this;
        }

        public Builder setRoleDuo() {
            this.isDuo = true;
            return this;
        }

        public Builder setRoleMaxCount(int count) {
            this.roleMaxCount = count;
            return this;
        }

        public Builder setRoleItems(List<ItemStack> roleItems) {
            this.roleItems = roleItems;
            return this;
        }

        public Builder addRoleItems(ItemStack ... roleItems) {
            this.roleItems.addAll(Arrays.asList(roleItems));
            return this;
        }

        public Builder setRoleEffects(List<Effect> roleEffects) {
            this.roleEffects = roleEffects;
            return this;
        }

        public Builder addRoleEffects(Effect... roleEffects) {
            this.roleEffects.addAll(Arrays.asList(roleEffects));
            return this;
        }

        public Builder addRoleEffect(Effect roleEffect) {
            this.roleEffects.add(roleEffect);
            return this;
        }

        public Builder setRoleProperties(Set<RoleProperties> roleProperties) {
            this.roleProperties = roleProperties;
            return this;
        }

        public Builder addRoleProperties(RoleProperties ... roleProperties) {
            this.roleProperties.addAll(List.of(roleProperties));
            return this;
        }

        public Role build() {
            return new Role(this);
        }
    }
}
